﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Security.Cryptography.X509Certificates;
using System.IO;

namespace iBSB2BTestClient
{
    public partial class Form1 : Form
    {

        private class Item
        {
            public string Name;
            public string ThumbPrint;
            public X509Certificate2 Cert;
            public Item(string name, string thumbPrint, X509Certificate2 cert)
            {
                Name = name; ThumbPrint = thumbPrint; Cert = cert;
            }
            public override string ToString()
            {
                // Generates the text shown in the combo box
                return Cert.Subject + ", " + Cert.NotAfter.ToString("dd.MM.yyyy");
            }
        }

        public Form1()
        {
            InitializeComponent();
        }
        private void ListUserCerts(ComboBox.ObjectCollection list)
        {
            X509Store my = new X509Store(StoreName.My, StoreLocation.CurrentUser);
            my.Open(OpenFlags.ReadOnly);
            foreach (X509Certificate2 cert in my.Certificates)
            {
                list.Add(new Item("", cert.Thumbprint, cert));
            }
        }

        private BSB2BWS.BS_B2BWSSoapClient PrepareWSCall()
        {
            if (cmbCert.SelectedItem == null)
            {
                MessageBox.Show("Prijavno digitalno potrdilo ni izbrano.");
                return null;
            }
            BSB2BWS.BS_B2BWSSoapClient soap = new BSB2BWS.BS_B2BWSSoapClient();
            soap.Endpoint.Address = new System.ServiceModel.EndpointAddress(txtAddress.Text);
            soap.ClientCredentials.ClientCertificate.Certificate = ((Item)cmbCert.SelectedItem).Cert;
            return soap;
        }

        private void setSenderPackageId()
        {

            txtSenderPackageId.Text = DateTime.Now.ToString("yyyyMMddhhmmssfff");
        }

        private void SetSubjArea(bool NeedSign, bool NeedZip, bool NeedEncryption)
        {
            chkSign.Checked = NeedSign;
            chkZip.Checked = NeedZip;
            chkEncrypt.Checked = NeedEncryption;
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            ListUserCerts(cmbCert.Items);
            ListUserCerts(cmbSignCert.Items);
            ListUserCerts(cmbCryptCert.Items);
            setSenderPackageId();
        }
       
        private bool SavePkg(string FileName, byte[] data)
        {
            using (SaveFileDialog sfd = new SaveFileDialog())
            {
                sfd.FileName = FileName;
                if (sfd.ShowDialog() != System.Windows.Forms.DialogResult.OK) return false;

                byte[] fdata = null;
                iCryptoLib cl = new iCryptoLib();
                
                if (chkEncrypt.Checked)
                {
                    fdata = cl.DecryptData(ref data);
                    if (cl.Error)
                    {
                        txtResp.Text = cl.ErrorMessage;
                        return false;
                    }
                }

                if (chkZip.Checked)
                {
                    if (fdata != null)
                        fdata = cl.UnZipData(ref fdata);
                    else
                        fdata = cl.UnZipData(ref data);
                    if (cl.Error)
                    {
                        txtResp.Text = cl.ErrorMessage;
                        return false;
                    }
                }
                if (chkSign.Checked)
                {
                    if (fdata != null)
                        cl.VerifyXmlSignature(fdata);
                    else
                        cl.VerifyXmlSignature(data);
                    if (cl.Error)
                    {
                        txtResp.Text = cl.ErrorMessage;
                        return false;
                    }
                }
                // glede na kljukice naredi ustrezno (dekriptiraj, odzipaj, preveri podpis)
                if (fdata != null)
                    File.WriteAllBytes(sfd.FileName, fdata);
                else
                    File.WriteAllBytes(sfd.FileName, data);
                return true;
            }
        }
        private byte[] PreparePkgContent()
        {
            byte[] data = null;
           iCryptoLib cl = new iCryptoLib();
            //cl.EncryptionOid = null;
            if (chkSign.Checked)
            {
                if (txtPackageName.Text.ToLower().EndsWith(".xml"))
                    data = cl.SignXmlDataST(txtFile.Text, "", ((Item)cmbSignCert.SelectedItem).Cert.Thumbprint);
                else if (txtPackageName.Text.ToLower().EndsWith(".pdf"))
                    data = cl.SignPDF(txtFile.Text, ((Item)cmbSignCert.SelectedItem).Cert.Thumbprint);
                if (cl.Error)
                {
                    throw new Exception(cl.ErrorMessage);
                }
            }
            if (chkZip.Checked)
            {
                if (data != null)
                    data = cl.ZipDataToZipFile(ref data, txtPackageName.Text);
                //data = cl.ZipData(ref data);
                else
                    data = cl.ZipDataToZipFile(txtFile.Text);
                //data = cl.ZipData(txtFile.Text);
                if (cl.Error)
                {
                    throw new Exception(cl.ErrorMessage);
                }
            }
            if (chkEncrypt.Checked)
            {
                //cl.CertStoreName = "I4";
                if (data != null)
                    data = cl.EncryptData(ref data, ((Item)cmbCryptCert.SelectedItem).Cert.Thumbprint);
                else
                    data = cl.EncryptData(txtFile.Text, ((Item)cmbCryptCert.SelectedItem).Cert.Thumbprint);
                if (cl.Error)
                {
                    throw new Exception(cl.ErrorMessage);
                }
            }
            if (data == null) data = File.ReadAllBytes(txtFile.Text);
            return data;
        }

        private void btnSendPackage_Click(object sender, EventArgs e)
        {
            if (chkSign.Checked && cmbSignCert.SelectedItem == null)
            {
                MessageBox.Show("Podpisno digitalno potrdilo ni izbrano.");
                return;
            }
            if (chkEncrypt.Checked && cmbCryptCert.SelectedItem == null)
            {
                MessageBox.Show("Kriptirno digitalno potrdilo ni izbrano.");
                return;
            }
            BSB2BWS.BS_B2BWSSoapClient soap = PrepareWSCall();
            if (soap == null) return;
            string stat = "", err = "";
            BSB2BWS.SendPackageRequestType req = new BSB2BWS.SendPackageRequestType();
            req.BSPackageType = txtBSPackageType.Text;
            req.SenderPackageId = txtSenderPackageId.Text;
            req.PackageName = txtPackageName.Text;
            req.PackageContent = PreparePkgContent();
            BSB2BWS.ResponseResultType resp = soap.SendPackage(req);
            stat = resp.StatusCode;
            err = resp.ErrorMessage;
            txtResp.Text = "ResponseStatus = " + stat;
            if (!String.IsNullOrEmpty(err))
                txtResp.Text = txtResp.Text + Environment.NewLine + "ErrorMessage = " + err;

            txtResp.Text = txtResp.Text + Environment.NewLine + Environment.NewLine;
            setSenderPackageId();

        }

        private void btnGetSubjectArea_Click(object sender, EventArgs e)
        {
            BSB2BWS.BS_B2BWSSoapClient soap = PrepareWSCall();
            if (soap == null) return;
            BSB2BWS.GetSubjectAreaResponseType resp = soap.GetSubjectArea(txtBSPkgType1.Text);
            string stat = "", err = "", subjArea = "";
            if (resp.SubjectAreaInfo != null)
            {
                System.Xml.Serialization.XmlSerializer x = new System.Xml.Serialization.XmlSerializer(resp.SubjectAreaInfo.GetType());
                StringWriter textWriter = new StringWriter();
                SetSubjArea(resp.SubjectAreaInfo.NeedSign, resp.SubjectAreaInfo.NeedZip, resp.SubjectAreaInfo.NeedEncryption);
                x.Serialize(textWriter, resp.SubjectAreaInfo);
                subjArea = textWriter.ToString();
            }
            stat = resp.ResponseStatus.StatusCode;
            err = resp.ResponseStatus.ErrorMessage;
            txtResp.Text = "ResponseStatus = " + stat;
            if (!String.IsNullOrEmpty(err))
                txtResp.Text = txtResp.Text + Environment.NewLine + "ErrorMessage = " + err;

            txtResp.Text = txtResp.Text + Environment.NewLine + Environment.NewLine + subjArea;
        }

        private void btnGetNewPkg_Click(object sender, EventArgs e)
        {
            string stat = "", err = "", str = "";
            BSB2BWS.BS_B2BWSSoapClient soap = PrepareWSCall();
            if (soap == null) return;
            BSB2BWS.GetPackageResponseType resp = soap.GetNewPackage(txtBSPkgType1.Text);
            stat = resp.ResponseStatus.StatusCode;
            err = resp.ResponseStatus.ErrorMessage;
            if (stat.Equals("OK"))
            {
                SavePkg((String.IsNullOrEmpty(resp.PackageName) ? resp.BSPackageId : resp.PackageName), resp.PackageContent);
                StringBuilder sb = new StringBuilder();
                sb.AppendLine("BSPackageId=" + resp.BSPackageId);
                sb.AppendLine("PackageName=" + resp.PackageName);
                sb.AppendLine("PackageSize=" + resp.PackageSize);
                sb.AppendLine("BSPackageType=" + resp.BSPackageType);
                sb.AppendLine("RestPackagesQuantity=" + resp.RestPackagesQuantity);
                str = sb.ToString();
            }

            txtResp.Text = "ResponseStatus = " + stat;
            if (!String.IsNullOrEmpty(err))
                txtResp.Text = txtResp.Text + Environment.NewLine + "ErrorMessage = " + err;

            txtResp.Text = txtResp.Text + Environment.NewLine + Environment.NewLine + str;
        }

        private void btnConfirmPkg_Click(object sender, EventArgs e)
        {
            string stat = "", err = "";
            BSB2BWS.BS_B2BWSSoapClient soap = PrepareWSCall();
            if (soap == null) return;
            BSB2BWS.ResponseResultType resp = soap.ConfirmPackage(txtBSPackageId.Text); ;
            stat = resp.StatusCode;
            err = resp.ErrorMessage;
            txtResp.Text = "ResponseStatus = " + stat;
            if (!String.IsNullOrEmpty(err))
                txtResp.Text = txtResp.Text + Environment.NewLine + "ErrorMessage = " + err;

            txtResp.Text = txtResp.Text + Environment.NewLine + Environment.NewLine;
        }

        private void btnBrowse_Click(object sender, EventArgs e)
        {

            using (OpenFileDialog fd = new OpenFileDialog())
            {
                if (fd.ShowDialog() != System.Windows.Forms.DialogResult.OK) return;
                txtFile.Text = fd.FileName;
                FileInfo fi = new FileInfo(fd.FileName);
                txtPackageSize.Text = fi.Length.ToString();
                txtPackageName.Text = fi.Name;
            }

        }
    }
}
